@@ -440,6 +440,13 @@ e_data_book_factory_class_init (EDataBookFactoryClass *class)
 	GObjectClass *object_class;
 	EDBusServerClass *dbus_server_class;
 	EDataFactoryClass *data_factory_class;
+	const gchar *modules_directory = BACKENDDIR;
+	const gchar *modules_directory_env;
+
+	modules_directory_env = g_getenv (EDS_ADDRESS_BOOK_MODULES);
+	if (modules_directory_env &&
+	    g_file_test (modules_directory_env, G_FILE_TEST_IS_DIR))
+		modules_directory = g_strdup (modules_directory_env);
 
 	g_type_class_add_private (class, sizeof (EDataBookFactoryPrivate));
 
@@ -450,7 +457,7 @@ e_data_book_factory_class_init (EDataBookFactoryClass *class)
 
 	dbus_server_class = E_DBUS_SERVER_CLASS (class);
 	dbus_server_class->bus_name = ADDRESS_BOOK_DBUS_SERVICE_NAME;
-	dbus_server_class->module_directory = BACKENDDIR;
+	dbus_server_class->module_directory = modules_directory;
 	dbus_server_class->bus_acquired = data_book_factory_bus_acquired;
 	dbus_server_class->bus_name_lost = data_book_factory_bus_name_lost;
 	dbus_server_class->quit_server = data_book_factory_quit_server;
