@@ -160,31 +160,23 @@ e_cal_backend_http_finalize (GObject *object)
 static void
 e_cal_backend_http_constructed (GObject *object)
 {
-	ESource *source;
 	ECalBackendHttp *backend;
-	ESourceWebdav *extension;
 	SoupSession *soup_session;
-	const gchar *extension_name;
 
 	/* Chain up to parent's constructed() method. */
 	G_OBJECT_CLASS (e_cal_backend_http_parent_class)->constructed (object);
 
 	soup_session = soup_session_sync_new ();
-	g_object_set (soup_session, SOUP_SESSION_TIMEOUT, 90, NULL);
+	g_object_set (
+		soup_session,
+		SOUP_SESSION_TIMEOUT, 90,
+		SOUP_SESSION_SSL_STRICT, TRUE,
+		SOUP_SESSION_SSL_USE_SYSTEM_CA_FILE, TRUE,
+		NULL);
 
 	backend = E_CAL_BACKEND_HTTP (object);
 	backend->priv->soup_session = soup_session;
 
-	source = e_backend_get_source (E_BACKEND (backend));
-	extension_name = E_SOURCE_EXTENSION_WEBDAV_BACKEND;
-	extension = e_source_get_extension (source, extension_name);
-
-	g_object_bind_property (
-		extension, "ignore-invalid-cert",
-		soup_session, SOUP_SESSION_SSL_USE_SYSTEM_CA_FILE,
-		G_BINDING_SYNC_CREATE |
-		G_BINDING_INVERT_BOOLEAN);
-
 	g_signal_connect (
 		backend->priv->soup_session, "authenticate",
 		G_CALLBACK (soup_authenticate), backend);
@@ -310,16 +302,11 @@ static icaltimezone *
 resolve_tzid (const gchar *tzid,
               gpointer user_data)
 {
-	icaltimezone *zone;
+	ETimezoneCache *timezone_cache;
 
-	zone = (!strcmp (tzid, "UTC"))
-		? icaltimezone_get_utc_timezone ()
-		: icaltimezone_get_builtin_timezone_from_tzid (tzid);
+	timezone_cache = E_TIMEZONE_CACHE (user_data);
 
-	if (!zone)
-		zone = e_cal_backend_internal_get_timezone (E_CAL_BACKEND (user_data), tzid);
-
-	return zone;
+	return e_timezone_cache_get_timezone (timezone_cache, tzid);
 }
 
 static gboolean
@@ -457,6 +444,7 @@ cal_backend_http_load (ECalBackendHttp *backend,
                        GError **error)
 {
 	ECalBackendHttpPrivate *priv = backend->priv;
+	ETimezoneCache *timezone_cache;
 	SoupMessage *soup_message;
 	SoupSession *soup_session;
 	icalcomponent *icalcomp, *subcomp;
@@ -473,6 +461,8 @@ cal_backend_http_load (ECalBackendHttp *backend,
 		SoupMessage *soup_message;
 	} cancel_data;
 
+	timezone_cache = E_TIMEZONE_CACHE (backend);
+
 	soup_session = backend->priv->soup_session;
 	soup_message = cal_backend_http_new_message (backend, uri);
 
@@ -495,6 +485,36 @@ cal_backend_http_load (ECalBackendHttp *backend,
 	}
 
 	status_code = soup_session_send_message (soup_session, soup_message);
+	if (status_code == SOUP_STATUS_SSL_FAILED) {
+		ESource *source;
+		ESourceWebdav *extension;
+		ESourceRegistry *registry;
+		EBackend *ebackend;
+		ETrustPromptResponse response;
+		ENamedParameters *parameters;
+
+		ebackend = E_BACKEND (backend);
+		source = e_backend_get_source (ebackend);
+		registry = e_cal_backend_get_registry (E_CAL_BACKEND (backend));
+		extension = e_source_get_extension (source, E_SOURCE_EXTENSION_WEBDAV_BACKEND);
+
+		parameters = e_named_parameters_new ();
+
+		response = e_source_webdav_prepare_ssl_trust_prompt (extension, soup_message, registry, parameters);
+		if (response == E_TRUST_PROMPT_RESPONSE_UNKNOWN) {
+			response = e_backend_trust_prompt_sync (ebackend, parameters, cancellable, NULL);
+			if (response != E_TRUST_PROMPT_RESPONSE_UNKNOWN)
+				e_source_webdav_store_ssl_trust_prompt (extension, soup_message, response);
+		}
+
+		e_named_parameters_free (parameters);
+
+		if (response == E_TRUST_PROMPT_RESPONSE_ACCEPT ||
+		    response == E_TRUST_PROMPT_RESPONSE_ACCEPT_TEMPORARILY) {
+			g_object_set (soup_session, SOUP_SESSION_SSL_STRICT, FALSE, NULL);
+			status_code = soup_session_send_message (soup_session, soup_message);
+		}
+	}
 
 	if (G_IS_CANCELLABLE (cancellable))
 		g_cancellable_disconnect (cancellable, cancel_id);
@@ -658,7 +678,7 @@ cal_backend_http_load (ECalBackendHttp *backend,
 
 			zone = icaltimezone_new ();
 			icaltimezone_set_component (zone, icalcomponent_new_clone (subcomp));
-			e_cal_backend_store_put_timezone (priv->store, (const icaltimezone *) zone);
+			e_timezone_cache_add_timezone (timezone_cache, zone);
 
 			icaltimezone_free (zone, 1);
 		}
@@ -783,6 +803,8 @@ source_changed_cb (ESource *source,
 {
 	g_return_if_fail (E_IS_CAL_BACKEND_HTTP (cbhttp));
 
+	g_object_ref (cbhttp);
+
 	if (cbhttp->priv->uri != NULL) {
 		gboolean uri_changed;
 		const gchar *new_uri;
@@ -806,6 +828,8 @@ source_changed_cb (ESource *source,
 
 		g_free (old_uri);
 	}
+
+	g_object_unref (cbhttp);
 }
 
 static void
@@ -839,6 +863,7 @@ e_cal_backend_http_open (ECalBackendSync *backend,
 	ESource *source;
 	ESourceRegistry *registry;
 	ESourceAuthentication *auth_extension;
+	ESourceWebdav *webdav_extension;
 	const gchar *extension_name;
 	const gchar *cache_dir;
 	gboolean auth_required;
@@ -865,6 +890,12 @@ e_cal_backend_http_open (ECalBackendSync *backend,
 	auth_extension = e_source_get_extension (source, extension_name);
 	auth_required = e_source_authentication_required (auth_extension);
 
+	extension_name = E_SOURCE_EXTENSION_WEBDAV_BACKEND;
+	webdav_extension = e_source_get_extension (source, extension_name);
+
+	g_object_set (cbhttp->priv->soup_session, SOUP_SESSION_SSL_STRICT, TRUE, NULL);
+	e_source_webdav_unset_temporary_ssl_trust (webdav_extension);
+
 	if (priv->source_changed_id == 0) {
 		priv->source_changed_id = g_signal_connect (
 			source, "changed",
@@ -879,7 +910,8 @@ e_cal_backend_http_open (ECalBackendSync *backend,
 	if (priv->store == NULL) {
 		/* remove the old cache while migrating to ECalBackendStore */
 		e_cal_backend_cache_remove (cache_dir, "cache.xml");
-		priv->store = e_cal_backend_file_store_new (cache_dir);
+		priv->store = e_cal_backend_store_new (
+			cache_dir, E_TIMEZONE_CACHE (backend));
 		e_cal_backend_store_load (priv->store);
 
 		if (!priv->store) {
@@ -1022,17 +1054,16 @@ e_cal_backend_http_add_timezone (ECalBackendSync *backend,
                                  GError **error)
 {
 	ECalBackendHttp *cbhttp;
-	ECalBackendHttpPrivate *priv;
+	ETimezoneCache *timezone_cache;
 	icalcomponent *tz_comp;
 	icaltimezone *zone;
 
 	cbhttp = (ECalBackendHttp *) backend;
+	timezone_cache = E_TIMEZONE_CACHE (backend);
 
 	e_return_data_cal_error_if_fail (E_IS_CAL_BACKEND_HTTP (cbhttp), InvalidArg);
 	e_return_data_cal_error_if_fail (tzobj != NULL, InvalidArg);
 
-	priv = cbhttp->priv;
-
 	tz_comp = icalparser_parse_string (tzobj);
 	if (!tz_comp) {
 		g_propagate_error (error, EDC_ERROR (InvalidObject));
@@ -1047,7 +1078,7 @@ e_cal_backend_http_add_timezone (ECalBackendSync *backend,
 
 	zone = icaltimezone_new ();
 	icaltimezone_set_component (zone, tz_comp);
-	e_cal_backend_store_put_timezone (priv->store, zone);
+	e_timezone_cache_add_timezone (timezone_cache, zone);
 }
 
 /* Get_objects_in_range handler for the file backend */
@@ -1063,12 +1094,15 @@ e_cal_backend_http_get_object_list (ECalBackendSync *backend,
 	ECalBackendHttpPrivate *priv;
 	GSList *components, *l;
 	ECalBackendSExp *cbsexp;
+	ETimezoneCache *timezone_cache;
 	time_t occur_start = -1, occur_end = -1;
 	gboolean prunning_by_time;
 
 	cbhttp = E_CAL_BACKEND_HTTP (backend);
 	priv = cbhttp->priv;
 
+	timezone_cache = E_TIMEZONE_CACHE (backend);
+
 	if (!priv->store) {
 		g_propagate_error (perror, EDC_ERROR (NoSuchCal));
 		return;
@@ -1088,7 +1122,7 @@ e_cal_backend_http_get_object_list (ECalBackendSync *backend,
 		: e_cal_backend_store_get_components (priv->store);
 
 	for (l = components; l != NULL; l = g_slist_next (l)) {
-		if (e_cal_backend_sexp_match_comp (cbsexp, E_CAL_COMPONENT (l->data), E_CAL_BACKEND (backend))) {
+		if (e_cal_backend_sexp_match_comp (cbsexp, E_CAL_COMPONENT (l->data), timezone_cache)) {
 			*objects = g_slist_append (*objects, e_cal_component_get_as_string (l->data));
 		}
 	}
@@ -1107,12 +1141,15 @@ e_cal_backend_http_start_view (ECalBackend *backend,
 	GSList *components, *l;
 	GSList *objects = NULL;
 	ECalBackendSExp *cbsexp;
+	ETimezoneCache *timezone_cache;
 	time_t occur_start = -1, occur_end = -1;
 	gboolean prunning_by_time;
 
 	cbhttp = E_CAL_BACKEND_HTTP (backend);
 	priv = cbhttp->priv;
 
+	timezone_cache = E_TIMEZONE_CACHE (backend);
+
 	cbsexp = e_data_cal_view_get_sexp (query);
 
 	d (g_message (G_STRLOC ": Starting query (%s)", e_cal_backend_sexp_text (cbsexp)));
@@ -1138,7 +1175,7 @@ e_cal_backend_http_start_view (ECalBackend *backend,
 	for (l = components; l != NULL; l = g_slist_next (l)) {
 		ECalComponent *comp = l->data;
 
-		if (e_cal_backend_sexp_match_comp (cbsexp, comp, E_CAL_BACKEND (backend))) {
+		if (e_cal_backend_sexp_match_comp (cbsexp, comp, timezone_cache)) {
 			objects = g_slist_append (objects, comp);
 		}
 	}
@@ -1267,7 +1304,9 @@ create_user_free_busy (ECalBackendHttp *cbhttp,
 				continue;
 		}
 
-		if (!e_cal_backend_sexp_match_comp (obj_sexp, l->data, E_CAL_BACKEND (cbhttp)))
+		if (!e_cal_backend_sexp_match_comp (
+			obj_sexp, l->data,
+			E_TIMEZONE_CACHE (cbhttp)))
 			continue;
 
 		vcalendar_comp = icalcomponent_get_parent (icalcomp);
@@ -1408,32 +1447,6 @@ e_cal_backend_http_send_objects (ECalBackendSync *backend,
 	g_propagate_error (perror, EDC_ERROR (PermissionDenied));
 }
 
-static icaltimezone *
-e_cal_backend_http_internal_get_timezone (ECalBackend *backend,
-                                          const gchar *tzid)
-{
-	ECalBackendHttp *cbhttp;
-	ECalBackendHttpPrivate *priv;
-	icaltimezone *zone;
-
-	cbhttp = E_CAL_BACKEND_HTTP (backend);
-	priv = cbhttp->priv;
-
-	g_return_val_if_fail (tzid != NULL, NULL);
-
-	if (!strcmp (tzid, "UTC"))
-		zone = icaltimezone_get_utc_timezone ();
-	else {
-		/* first try to get the timezone from the cache */
-		zone = (icaltimezone *) e_cal_backend_store_get_timezone (priv->store, tzid);
-
-		if (!zone && E_CAL_BACKEND_CLASS (e_cal_backend_http_parent_class)->internal_get_timezone)
-			zone = E_CAL_BACKEND_CLASS (e_cal_backend_http_parent_class)->internal_get_timezone (backend, tzid);
-	}
-
-	return zone;
-}
-
 static ESourceAuthenticationResult
 cal_backend_http_try_password_sync (ESourceAuthenticator *authenticator,
                                     const GString *password,
@@ -1511,7 +1524,6 @@ e_cal_backend_http_class_init (ECalBackendHttpClass *class)
 	sync_class->get_free_busy_sync		= e_cal_backend_http_get_free_busy;
 
 	backend_class->start_view		= e_cal_backend_http_start_view;
-	backend_class->internal_get_timezone	= e_cal_backend_http_internal_get_timezone;
 }
 
 static void
