@@ -54,8 +54,6 @@ struct _ECalBackendWeatherPrivate {
 	/* The file cache */
 	ECalBackendStore *store;
 
-	GHashTable *zones;
-
 	/* Reload */
 	guint reload_timeout_id;
 	guint is_loading : 1;
@@ -129,16 +127,11 @@ static icaltimezone *
 resolve_tzid (const gchar *tzid,
               gpointer user_data)
 {
-	icaltimezone *zone;
+	ETimezoneCache *timezone_cache;
 
-	zone = (!strcmp (tzid, "UTC"))
-		? icaltimezone_get_utc_timezone ()
-		: icaltimezone_get_builtin_timezone_from_tzid (tzid);
+	timezone_cache = E_TIMEZONE_CACHE (user_data);
 
-	if (!zone)
-		zone = e_cal_backend_internal_get_timezone (E_CAL_BACKEND (user_data), tzid);
-
-	return zone;
+	return e_timezone_cache_get_timezone (timezone_cache, tzid);
 }
 
 static void
@@ -506,7 +499,8 @@ e_cal_backend_weather_open (ECalBackendSync *backend,
 
 	if (!priv->store) {
 		e_cal_backend_cache_remove (cache_dir, "cache.xml");
-		priv->store = e_cal_backend_file_store_new (cache_dir);
+		priv->store = e_cal_backend_store_new (
+			cache_dir, E_TIMEZONE_CACHE (backend));
 
 		if (!priv->store) {
 			g_propagate_error (perror, EDC_ERROR_EX (OtherError, _("Could not create cache file")));
@@ -597,16 +591,20 @@ e_cal_backend_weather_get_object_list (ECalBackendSync *backend,
 {
 	ECalBackendWeather *cbw = E_CAL_BACKEND_WEATHER (backend);
 	ECalBackendWeatherPrivate *priv = cbw->priv;
-	ECalBackendSExp *sexp = e_cal_backend_sexp_new (sexp_string);
+	ECalBackendSExp *sexp;
+	ETimezoneCache *timezone_cache;
 	GSList *components, *l;
 	time_t occur_start = -1, occur_end = -1;
 	gboolean prunning_by_time;
 
-	if (!sexp) {
+	sexp = e_cal_backend_sexp_new (sexp_string);
+	if (sexp == NULL) {
 		g_propagate_error (perror, EDC_ERROR (InvalidQuery));
 		return;
 	}
 
+	timezone_cache = E_TIMEZONE_CACHE (backend);
+
 	*objects = NULL;
 	prunning_by_time = e_cal_backend_sexp_evaluate_occur_times (
 		sexp,
@@ -618,7 +616,7 @@ e_cal_backend_weather_get_object_list (ECalBackendSync *backend,
 		: e_cal_backend_store_get_components (priv->store);
 
 	for (l = components; l != NULL; l = g_slist_next (l)) {
-		if (e_cal_backend_sexp_match_comp (sexp, E_CAL_COMPONENT (l->data), E_CAL_BACKEND (backend)))
+		if (e_cal_backend_sexp_match_comp (sexp, E_CAL_COMPONENT (l->data), timezone_cache))
 			*objects = g_slist_append (*objects, e_cal_component_get_as_string (l->data));
 	}
 
@@ -635,18 +633,14 @@ e_cal_backend_weather_add_timezone (ECalBackendSync *backend,
                                     GError **error)
 {
 	ECalBackendWeather *cbw;
-	ECalBackendWeatherPrivate *priv;
 	icalcomponent *tz_comp;
 	icaltimezone *zone;
-	const gchar *tzid;
 
 	cbw = (ECalBackendWeather *) backend;
 
 	e_return_data_cal_error_if_fail (E_IS_CAL_BACKEND_WEATHER (cbw), InvalidArg);
 	e_return_data_cal_error_if_fail (tzobj != NULL, InvalidArg);
 
-	priv = cbw->priv;
-
 	tz_comp = icalparser_parse_string (tzobj);
 	e_return_data_cal_error_if_fail (tz_comp != NULL, InvalidObject);
 
@@ -657,13 +651,8 @@ e_cal_backend_weather_add_timezone (ECalBackendSync *backend,
 
 	zone = icaltimezone_new ();
 	icaltimezone_set_component (zone, tz_comp);
-	tzid = icaltimezone_get_tzid (zone);
-
-	if (g_hash_table_lookup (priv->zones, tzid)) {
-		icaltimezone_free (zone, TRUE);
-	} else {
-		g_hash_table_insert (priv->zones, g_strdup (tzid), zone);
-	}
+	e_timezone_cache_add_timezone (E_TIMEZONE_CACHE (backend), zone);
+	icaltimezone_free (zone, TRUE);
 }
 
 static void
@@ -696,6 +685,7 @@ e_cal_backend_weather_start_view (ECalBackend *backend,
 	ECalBackendWeather *cbw;
 	ECalBackendWeatherPrivate *priv;
 	ECalBackendSExp *sexp;
+	ETimezoneCache *timezone_cache;
 	GSList *components, *l;
 	GSList *objects;
 	GError *error;
@@ -720,6 +710,8 @@ e_cal_backend_weather_start_view (ECalBackend *backend,
 		return;
 	}
 
+	timezone_cache = E_TIMEZONE_CACHE (backend);
+
 	objects = NULL;
 	prunning_by_time = e_cal_backend_sexp_evaluate_occur_times (sexp, &occur_start, &occur_end);
 	components = prunning_by_time ?
@@ -727,7 +719,7 @@ e_cal_backend_weather_start_view (ECalBackend *backend,
 		: e_cal_backend_store_get_components (priv->store);
 
 	for (l = components; l != NULL; l = g_slist_next (l)) {
-		if (e_cal_backend_sexp_match_comp (sexp, E_CAL_COMPONENT (l->data), backend))
+		if (e_cal_backend_sexp_match_comp (sexp, E_CAL_COMPONENT (l->data), timezone_cache))
 			objects = g_slist_prepend (objects, l->data);
 	}
 
@@ -766,36 +758,6 @@ e_cal_backend_weather_notify_online_cb (ECalBackend *backend,
 	}
 }
 
-static icaltimezone *
-e_cal_backend_weather_internal_get_timezone (ECalBackend *backend,
-                                             const gchar *tzid)
-{
-	icaltimezone *zone;
-
-	g_return_val_if_fail (tzid != NULL, NULL);
-
-	if (!strcmp (tzid, "UTC")) {
-		zone = icaltimezone_get_utc_timezone ();
-	} else {
-		ECalBackendWeather *cbw = E_CAL_BACKEND_WEATHER (backend);
-
-		g_return_val_if_fail (E_IS_CAL_BACKEND_WEATHER (cbw), NULL);
-
-		zone = g_hash_table_lookup (cbw->priv->zones, tzid);
-
-		if (!zone && E_CAL_BACKEND_CLASS (e_cal_backend_weather_parent_class)->internal_get_timezone)
-			zone = E_CAL_BACKEND_CLASS (e_cal_backend_weather_parent_class)->internal_get_timezone (backend, tzid);
-	}
-
-	return zone;
-}
-
-static void
-free_zone (gpointer data)
-{
-	icaltimezone_free (data, TRUE);
-}
-
 /* Finalize handler for the weather backend */
 static void
 e_cal_backend_weather_finalize (GObject *object)
@@ -815,8 +777,6 @@ e_cal_backend_weather_finalize (GObject *object)
 		priv->store = NULL;
 	}
 
-	g_hash_table_destroy (priv->zones);
-
 	g_free (priv->city);
 
 	/* Chain up to parent's finalize() method. */
@@ -829,12 +789,6 @@ e_cal_backend_weather_init (ECalBackendWeather *cbw)
 {
 	cbw->priv = E_CAL_BACKEND_WEATHER_GET_PRIVATE (cbw);
 
-	cbw->priv->zones = g_hash_table_new_full (
-		(GHashFunc) g_str_hash,
-		(GEqualFunc) g_str_equal,
-		(GDestroyNotify) g_free,
-		(GDestroyNotify) free_zone);
-
 	e_cal_backend_sync_set_lock (E_CAL_BACKEND_SYNC (cbw), TRUE);
 
 	g_signal_connect (
@@ -868,7 +822,6 @@ e_cal_backend_weather_class_init (ECalBackendWeatherClass *class)
 	sync_class->get_free_busy_sync		= e_cal_backend_weather_get_free_busy;
 
 	backend_class->start_view		= e_cal_backend_weather_start_view;
-	backend_class->internal_get_timezone	= e_cal_backend_weather_internal_get_timezone;
 
 	/* Register our ESource extension. */
 	E_TYPE_SOURCE_WEATHER;
