@@ -115,6 +115,49 @@ struct _EDataCalClass {
 	GObjectClass parent_class;
 };
 
+/**
+ * EDataCalCallStatus:
+ *
+ * FIXME Document me!
+ *
+ * Since: 3.6
+ **/
+typedef enum {
+	Success,
+	Busy,
+	RepositoryOffline,
+	PermissionDenied,
+	InvalidRange,
+	ObjectNotFound,
+	InvalidObject,
+	ObjectIdAlreadyExists,
+	AuthenticationFailed,
+	AuthenticationRequired,
+	UnsupportedField,
+	UnsupportedMethod,
+	UnsupportedAuthenticationMethod,
+	TLSNotAvailable,
+	NoSuchCal,
+	UnknownUser,
+	OfflineUnavailable,
+
+	/* These can be returned for successful searches, but
+		indicate the result set was truncated */
+	SearchSizeLimitExceeded,
+	SearchTimeLimitExceeded,
+
+	InvalidQuery,
+	QueryRefused,
+
+	CouldNotCancel,
+
+	OtherError,
+	InvalidServerVersion,
+	InvalidArg,
+	NotSupported,
+	NotOpened
+} EDataCalCallStatus;
+
 GQuark		e_data_cal_error_quark		(void);
 GError *	e_data_cal_create_error		(EDataCalCallStatus status,
 						 const gchar *custom_msg);
