@@ -37,6 +37,8 @@
 #include "camel-mime-utils.h"
 #include "camel-medium.h"
 #include "camel-multipart.h"
+#include "camel-multipart-encrypted.h"
+#include "camel-multipart-signed.h"
 #include "camel-mime-message.h"
 #include "camel-mime-filter-canon.h"
 #include "camel-stream-filter.h"
@@ -1814,6 +1816,10 @@ cc_prepare_sign (CamelMimePart *part)
 	if (!dw)
 		return;
 
+	/* should not change encoding for these, they have the right encoding set already */
+	if (CAMEL_IS_MULTIPART_SIGNED (dw) || CAMEL_IS_MULTIPART_ENCRYPTED (dw))
+		return;
+
 	if (CAMEL_IS_MULTIPART (dw)) {
 		parts = camel_multipart_get_number ((CamelMultipart *) dw);
 		for (i = 0; i < parts; i++)
