@@ -113,14 +113,17 @@ cdf_folder_changed (CamelFolder *folder,
 
 	service = CAMEL_SERVICE (parent_store);
 	session = camel_service_get_session (service);
-	settings = camel_service_get_settings (service);
 
 	sync_folder = camel_disco_folder_get_offline_sync (
 		CAMEL_DISCO_FOLDER (folder));
 
+	settings = camel_service_ref_settings (service);
+
 	sync_store = camel_offline_settings_get_stay_synchronized (
 		CAMEL_OFFLINE_SETTINGS (settings));
 
+	g_object_unref (settings);
+
 	if (changes->uid_added->len > 0 && (sync_folder || sync_store)) {
 		struct _cdf_sync_data *data;
 
@@ -524,7 +527,7 @@ camel_disco_folder_set_offline_sync (CamelDiscoFolder *disco_folder,
 {
 	g_return_if_fail (CAMEL_IS_DISCO_FOLDER (disco_folder));
 
-	if ((disco_folder->priv->offline_sync ? 1 : 0) == (offline_sync ? 1 : 0))
+	if (disco_folder->priv->offline_sync == offline_sync)
 		return;
 
 	disco_folder->priv->offline_sync = offline_sync;
