@@ -116,13 +116,13 @@ struct _CamelSessionClass {
 	gint		(*alert_user)		(CamelSession *session,
 						 CamelSessionAlertType type,
 						 const gchar *prompt,
-						 GSList *button_captions,
+						 GList *button_captions,
 						 GCancellable *cancellable);
 	CamelCertTrust	(*trust_prompt)		(CamelSession *session,
 						 const gchar *host,
 						 const gchar *certificate,
 						 guint32 certificate_errors,
-						 const GSList *issuers,
+						 GList *issuers,
 						 GCancellable *cancellable);
 	CamelFilterDriver *
 			(*get_filter_driver)	(CamelSession *session,
@@ -216,13 +216,13 @@ gboolean	camel_session_forget_password	(CamelSession *session,
 gint		camel_session_alert_user	(CamelSession *session,
 						 CamelSessionAlertType type,
 						 const gchar *prompt,
-						 GSList *button_captions,
+						 GList *button_captions,
 						 GCancellable *cancellable);
 CamelCertTrust	camel_session_trust_prompt	(CamelSession *session,
 						 const gchar *host,
 						 const gchar *certificate,
 						 guint32 certificate_errors,
-						 const GSList *issuers,
+						 GList *issuers,
 						 GCancellable *cancellable);
 gchar *		camel_session_build_password_prompt
 						(const gchar *type,
