@@ -55,7 +55,7 @@ typedef struct _AsyncContext AsyncContext;
 typedef struct _SignalData SignalData;
 
 struct _CamelStorePrivate {
-	GStaticRecMutex folder_lock;	/* for locking folder operations */
+	GRecMutex folder_lock;	/* for locking folder operations */
 };
 
 struct _AsyncContext {
@@ -261,7 +261,7 @@ store_finalize (GObject *object)
 	if (store->folders != NULL)
 		camel_object_bag_destroy (store->folders);
 
-	g_static_rec_mutex_free (&store->priv->folder_lock);
+	g_rec_mutex_clear (&store->priv->folder_lock);
 
 	if (store->cdb_r != NULL) {
 		camel_db_close (store->cdb_r);
@@ -343,8 +343,9 @@ store_synchronize_sync (CamelStore *store,
                         GError **error)
 {
 	GPtrArray *folders;
+	CamelFolder *folder;
 	gboolean success = TRUE;
-	gint ii;
+	gint i;
 	GError *local_error = NULL;
 
 	if (expunge) {
@@ -352,33 +353,27 @@ store_synchronize_sync (CamelStore *store,
 		CamelFolderInfo *root, *fi;
 
 		folders = g_ptr_array_new ();
-		root = camel_store_get_folder_info_sync (
-			store, NULL,
-			CAMEL_STORE_FOLDER_INFO_RECURSIVE |
-			CAMEL_STORE_FOLDER_INFO_SUBSCRIBED |
-			CAMEL_STORE_FOLDER_INFO_NO_VIRTUAL,
-			NULL, NULL);
+		root = camel_store_get_folder_info_sync (store, NULL, CAMEL_STORE_FOLDER_INFO_RECURSIVE | CAMEL_STORE_FOLDER_INFO_NO_VIRTUAL, NULL, NULL);
 		fi = root;
-		while (fi != NULL) {
+		while (fi) {
 			CamelFolderInfo *next;
 
 			if ((fi->flags & CAMEL_FOLDER_NOSELECT) == 0) {
-				CamelFolder *folder;
+				CamelFolder *fldr;
 
-				folder = camel_store_get_folder_sync (
-					store, fi->full_name, 0, NULL, NULL);
-				if (folder != NULL)
-					g_ptr_array_add (folders, folder);
+				fldr = camel_store_get_folder_sync (store, fi->full_name, 0, NULL, NULL);
+				if (fldr)
+					g_ptr_array_add (folders, fldr);
 			}
 
 			/* pick the next */
 			next = fi->child;
-			if (next == NULL)
+			if (!next)
 				next = fi->next;
-			if (next == NULL) {
+			if (!next) {
 				next = fi->parent;
-				while (next != NULL) {
-					if (next->next != NULL) {
+				while (next) {
+					if (next->next) {
 						next = next->next;
 						break;
 					}
@@ -390,7 +385,7 @@ store_synchronize_sync (CamelStore *store,
 			fi = next;
 		}
 
-		if (root != NULL)
+		if (root)
 			camel_store_free_folder_info_full (store, root);
 	} else {
 		/* sync only folders opened until now */
@@ -400,10 +395,10 @@ store_synchronize_sync (CamelStore *store,
 	/* We don't sync any vFolders, that is used to update certain
 	 * vfolder queries mainly, and we're really only interested in
 	 * storing/expunging the physical mails. */
-	for (ii = 0; ii < folders->len; ii++) {
-		CamelFolder *folder = folders->pdata[ii];
-
-		if (!CAMEL_IS_VEE_FOLDER (folder) && local_error == NULL) {
+	for (i = 0; i < folders->len; i++) {
+		folder = folders->pdata[i];
+		if (!CAMEL_IS_VEE_FOLDER (folder)
+		    && local_error == NULL) {
 			camel_folder_synchronize_sync (
 				folder, expunge, cancellable, &local_error);
 			ignore_no_such_table_exception (&local_error);
@@ -1246,7 +1241,7 @@ camel_store_init (CamelStore *store)
 
 	store->mode = CAMEL_STORE_READ | CAMEL_STORE_WRITE;
 
-	g_static_rec_mutex_init (&store->priv->folder_lock);
+	g_rec_mutex_init (&store->priv->folder_lock);
 }
 
 GQuark
@@ -1268,7 +1263,7 @@ camel_store_error_quark (void)
  * @folder_info: information about the created folder
  *
  * Emits the #CamelStore::folder-created signal from an idle source on
- * the main loop.  The idle source's priority is #G_PRIORITY_HIGH_IDLE.
+ * the main loop.  The idle source's priority is #G_PRIORITY_DEFAULT_IDLE.
  *
  * This function is only intended for Camel providers.
  *
@@ -1290,9 +1285,8 @@ camel_store_folder_created (CamelStore *store,
 	signal_data->store = g_object_ref (store);
 	signal_data->folder_info = camel_folder_info_clone (folder_info);
 
-	/* Prioritize ahead of GTK+ redraws. */
 	camel_session_idle_add (
-		session, G_PRIORITY_HIGH_IDLE,
+		session, G_PRIORITY_DEFAULT_IDLE,
 		store_emit_folder_created_cb,
 		signal_data, (GDestroyNotify) signal_data_free);
 }
@@ -1303,7 +1297,7 @@ camel_store_folder_created (CamelStore *store,
  * @folder_info: information about the deleted folder
  *
  * Emits the #CamelStore::folder-deleted signal from an idle source on
- * the main loop.  The idle source's priority is #G_PRIORITY_HIGH_IDLE.
+ * the main loop.  The idle source's priority is #G_PRIORITY_DEFAULT_IDLE.
  *
  * This function is only intended for Camel providers.
  *
@@ -1325,9 +1319,8 @@ camel_store_folder_deleted (CamelStore *store,
 	signal_data->store = g_object_ref (store);
 	signal_data->folder_info = camel_folder_info_clone (folder_info);
 
-	/* Prioritize ahead of GTK+ redraws. */
 	camel_session_idle_add (
-		session, G_PRIORITY_HIGH_IDLE,
+		session, G_PRIORITY_DEFAULT_IDLE,
 		store_emit_folder_deleted_cb,
 		signal_data, (GDestroyNotify) signal_data_free);
 }
@@ -1338,7 +1331,7 @@ camel_store_folder_deleted (CamelStore *store,
  * @folder: the #CamelFolder that was opened
  *
  * Emits the #CamelStore::folder-opened signal from an idle source on
- * the main loop.  The idle source's priority is #G_PRIORITY_HIGH_IDLE.
+ * the main loop.  The idle source's priority is #G_PRIORITY_DEFAULT_IDLE.
  *
  * This function is only intended for Camel providers.
  *
@@ -1360,9 +1353,8 @@ camel_store_folder_opened (CamelStore *store,
 	signal_data->store = g_object_ref (store);
 	signal_data->folder = g_object_ref (folder);
 
-	/* Prioritize ahead of GTK+ redraws. */
 	camel_session_idle_add (
-		session, G_PRIORITY_HIGH_IDLE,
+		session, G_PRIORITY_DEFAULT_IDLE,
 		store_emit_folder_opened_cb,
 		signal_data, (GDestroyNotify) signal_data_free);
 }
@@ -1374,7 +1366,7 @@ camel_store_folder_opened (CamelStore *store,
  * @folder_info: information about the renamed folder
  *
  * Emits the #CamelStore::folder-renamed signal from an idle source on
- * the main loop.  The idle source's priority is #G_PRIORITY_HIGH_IDLE.
+ * the main loop.  The idle source's priority is #G_PRIORITY_DEFAULT_IDLE.
  *
  * This function is only intended for Camel providers.
  *
@@ -1399,9 +1391,8 @@ camel_store_folder_renamed (CamelStore *store,
 	signal_data->folder_info = camel_folder_info_clone (folder_info);
 	signal_data->folder_name = g_strdup (old_name);
 
-	/* Prioritize ahead of GTK+ redraws. */
 	camel_session_idle_add (
-		session, G_PRIORITY_HIGH_IDLE,
+		session, G_PRIORITY_DEFAULT_IDLE,
 		store_emit_folder_renamed_cb,
 		signal_data, (GDestroyNotify) signal_data_free);
 }
@@ -1774,7 +1765,7 @@ camel_store_lock (CamelStore *store,
 
 	switch (lock) {
 		case CAMEL_STORE_FOLDER_LOCK:
-			g_static_rec_mutex_lock (&store->priv->folder_lock);
+			g_rec_mutex_lock (&store->priv->folder_lock);
 			break;
 		default:
 			g_return_if_reached ();
@@ -1798,7 +1789,7 @@ camel_store_unlock (CamelStore *store,
 
 	switch (lock) {
 		case CAMEL_STORE_FOLDER_LOCK:
-			g_static_rec_mutex_unlock (&store->priv->folder_lock);
+			g_rec_mutex_unlock (&store->priv->folder_lock);
 			break;
 		default:
 			g_return_if_reached ();
