@@ -81,7 +81,7 @@ struct _EAuthenticationSessionPrivate {
 	gchar *source_uid;
 
 	/* These are for configuring system prompts. */
-	GMutex *property_lock;
+	GMutex property_lock;
 	gchar *prompt_title;
 	gchar *prompt_message;
 	gchar *prompt_description;
@@ -313,7 +313,7 @@ authentication_session_finalize (GObject *object)
 
 	priv = E_AUTHENTICATION_SESSION_GET_PRIVATE (object);
 
-	g_mutex_free (priv->property_lock);
+	g_mutex_clear (&priv->property_lock);
 
 	g_free (priv->source_uid);
 	g_free (priv->prompt_title);
@@ -570,16 +570,30 @@ try_again:
 
 	if (auth_result == E_SOURCE_AUTHENTICATION_ACCEPTED) {
 		gboolean permanently;
+		gchar *password_copy;
 
 		permanently = gcr_prompt_get_choice_chosen (prompt);
 		session_result = E_AUTHENTICATION_SESSION_SUCCESS;
 
+		/* Close our prompt before storing the password in
+		 * the keyring.  If the keyring is locked, it will
+		 * need to prompt the user for a keyring password,
+		 * but it can't do that if our password prompt is
+		 * still open since both prompts are system-modal.
+		 * Not sure what would happen next; probably the
+		 * store operation would either fail or deadlock. */
+
+		/* XXX Not sure if it's safe to use the prompt's
+		 *     password string after closing the prompt,
+		 *     so make a copy here just to be safe. */
+		password_copy = gcr_secure_memory_strdup (prompt_password);
+
 		/* Failure here does not affect the outcome of this
 		 * operation, but leave a breadcrumb as evidence that
 		 * something went wrong. */
 
-		e_authentication_session_store_password_sync (
-			session, prompt_password, permanently,
+		gcr_system_prompt_close (
+			GCR_SYSTEM_PROMPT (prompt),
 			cancellable, &local_error);
 
 		if (local_error != NULL) {
@@ -587,7 +601,20 @@ try_again:
 			g_clear_error (&local_error);
 		}
 
-		goto close_prompt;
+		g_object_unref (prompt);
+
+		e_authentication_session_store_password_sync (
+			session, password_copy, permanently,
+			cancellable, &local_error);
+
+		if (local_error != NULL) {
+			g_warning ("%s: %s", G_STRFUNC, local_error->message);
+			g_clear_error (&local_error);
+		}
+
+		gcr_secure_memory_strfree (password_copy);
+
+		goto exit;
 	}
 
 	g_warn_if_fail (auth_result == E_SOURCE_AUTHENTICATION_REJECTED);
@@ -785,7 +812,7 @@ static void
 e_authentication_session_init (EAuthenticationSession *session)
 {
 	session->priv = E_AUTHENTICATION_SESSION_GET_PRIVATE (session);
-	session->priv->property_lock = g_mutex_new ();
+	g_mutex_init (&session->priv->property_lock);
 }
 
 GQuark
@@ -932,12 +959,12 @@ e_authentication_session_dup_prompt_title (EAuthenticationSession *session)
 
 	g_return_val_if_fail (E_IS_AUTHENTICATION_SESSION (session), NULL);
 
-	g_mutex_lock (session->priv->property_lock);
+	g_mutex_lock (&session->priv->property_lock);
 
 	protected = e_authentication_session_get_prompt_title (session);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (session->priv->property_lock);
+	g_mutex_unlock (&session->priv->property_lock);
 
 	return duplicate;
 }
@@ -958,17 +985,17 @@ e_authentication_session_set_prompt_title (EAuthenticationSession *session,
 {
 	g_return_if_fail (E_IS_AUTHENTICATION_SESSION (session));
 
-	g_mutex_lock (session->priv->property_lock);
+	g_mutex_lock (&session->priv->property_lock);
 
 	if (g_strcmp0 (session->priv->prompt_title, prompt_title) == 0) {
-		g_mutex_unlock (session->priv->property_lock);
+		g_mutex_unlock (&session->priv->property_lock);
 		return;
 	}
 
 	g_free (session->priv->prompt_title);
 	session->priv->prompt_title = g_strdup (prompt_title);
 
-	g_mutex_unlock (session->priv->property_lock);
+	g_mutex_unlock (&session->priv->property_lock);
 
 	g_object_notify (G_OBJECT (session), "prompt-title");
 }
@@ -1013,12 +1040,12 @@ e_authentication_session_dup_prompt_message (EAuthenticationSession *session)
 
 	g_return_val_if_fail (E_IS_AUTHENTICATION_SESSION (session), NULL);
 
-	g_mutex_lock (session->priv->property_lock);
+	g_mutex_lock (&session->priv->property_lock);
 
 	protected = e_authentication_session_get_prompt_message (session);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (session->priv->property_lock);
+	g_mutex_unlock (&session->priv->property_lock);
 
 	return duplicate;
 }
@@ -1039,17 +1066,17 @@ e_authentication_session_set_prompt_message (EAuthenticationSession *session,
 {
 	g_return_if_fail (E_IS_AUTHENTICATION_SESSION (session));
 
-	g_mutex_lock (session->priv->property_lock);
+	g_mutex_lock (&session->priv->property_lock);
 
 	if (g_strcmp0 (session->priv->prompt_message, prompt_message) == 0) {
-		g_mutex_unlock (session->priv->property_lock);
+		g_mutex_unlock (&session->priv->property_lock);
 		return;
 	}
 
 	g_free (session->priv->prompt_message);
 	session->priv->prompt_message = g_strdup (prompt_message);
 
-	g_mutex_unlock (session->priv->property_lock);
+	g_mutex_unlock (&session->priv->property_lock);
 
 	g_object_notify (G_OBJECT (session), "prompt-message");
 }
@@ -1095,12 +1122,12 @@ e_authentication_session_dup_prompt_description (EAuthenticationSession *session
 
 	g_return_val_if_fail (E_IS_AUTHENTICATION_SESSION (session), NULL);
 
-	g_mutex_lock (session->priv->property_lock);
+	g_mutex_lock (&session->priv->property_lock);
 
 	protected = e_authentication_session_get_prompt_description (session);
 	duplicate = g_strdup (protected);
 
-	g_mutex_unlock (session->priv->property_lock);
+	g_mutex_unlock (&session->priv->property_lock);
 
 	return duplicate;
 }
@@ -1121,17 +1148,17 @@ e_authentication_session_set_prompt_description (EAuthenticationSession *session
 {
 	g_return_if_fail (E_IS_AUTHENTICATION_SESSION (session));
 
-	g_mutex_lock (session->priv->property_lock);
+	g_mutex_lock (&session->priv->property_lock);
 
 	if (g_strcmp0 (session->priv->prompt_description, prompt_description) == 0) {
-		g_mutex_unlock (session->priv->property_lock);
+		g_mutex_unlock (&session->priv->property_lock);
 		return;
 	}
 
 	g_free (session->priv->prompt_description);
 	session->priv->prompt_description = g_strdup (prompt_description);
 
-	g_mutex_unlock (session->priv->property_lock);
+	g_mutex_unlock (&session->priv->property_lock);
 
 	g_object_notify (G_OBJECT (session), "prompt-description");
 }
