@@ -91,6 +91,12 @@ struct _ESourceRegistryServerClass {
 						 ESource *source);
 	void		(*source_removed)	(ESourceRegistryServer *server,
 						 ESource *source);
+	gboolean	(*tweak_key_file)	(ESourceRegistryServer *server,
+						 GKeyFile *key_file,
+						 const gchar *uid);
+
+	/* Reserved slots. */
+	gpointer reserved[16];
 };
 
 GType		e_source_registry_server_get_type
@@ -125,6 +131,10 @@ ESource *	e_source_registry_server_ref_source
 GList *		e_source_registry_server_list_sources
 						(ESourceRegistryServer *server,
 						 const gchar *extension_name);
+ESource *	e_source_registry_server_find_extension
+						(ESourceRegistryServer *server,
+						 ESource *source,
+						 const gchar *extension_name);
 ECollectionBackend *
 		e_source_registry_server_ref_backend
 						(ESourceRegistryServer *server,
