@@ -661,7 +661,7 @@ e_client_set_readonly (EClient *client,
 	g_return_if_fail (E_IS_CLIENT (client));
 
 	g_rec_mutex_lock (&client->priv->prop_mutex);
-	if ((readonly ? 1 : 0) == (client->priv->readonly ? 1 : 0)) {
+	if (client->priv->readonly == readonly) {
 		g_rec_mutex_unlock (&client->priv->prop_mutex);
 		return;
 	}
@@ -701,7 +701,7 @@ e_client_set_online (EClient *client,
 	e_client_set_capabilities (client, NULL);
 
 	g_rec_mutex_lock (&client->priv->prop_mutex);
-	if ((is_online ? 1: 0) == (client->priv->online ? 1 : 0)) {
+	if (client->priv->online == is_online) {
 		g_rec_mutex_unlock (&client->priv->prop_mutex);
 		return;
 	}
@@ -1604,6 +1604,8 @@ e_client_refresh_sync (EClient *client,
  * Note: Paired function for this is e_client_util_strv_to_slist().
  *
  * Since: 3.2
+ *
+ * Deprecated: 3.8: Use e_util_slist_to_strv() instead.
  **/
 gchar **
 e_client_util_slist_to_strv (const GSList *strings)
@@ -1624,6 +1626,8 @@ e_client_util_slist_to_strv (const GSList *strings)
  * Note: Paired function for this is e_client_util_slist_to_strv().
  *
  * Since: 3.2
+ *
+ * Deprecated: 3.8: Use e_util_strv_to_slist() instead.
  **/
 GSList *
 e_client_util_strv_to_slist (const gchar * const *strv)
@@ -1642,6 +1646,8 @@ e_client_util_strv_to_slist (const gchar * const *strv)
  * The returned pointer can be freed with e_client_util_free_string_slist().
  *
  * Since: 3.2
+ *
+ * Deprecated: 3.8: Use e_util_copy_string_slist() instead.
  **/
 GSList *
 e_client_util_copy_string_slist (GSList *copy_to,
@@ -1661,6 +1667,8 @@ e_client_util_copy_string_slist (GSList *copy_to,
  * The returned pointer can be freed with e_client_util_free_object_slist().
  *
  * Since: 3.2
+ *
+ * Deprecated: 3.8: Use e_util_copy_object_slist() instead.
  **/
 GSList *
 e_client_util_copy_object_slist (GSList *copy_to,
@@ -1676,6 +1684,8 @@ e_client_util_copy_object_slist (GSList *copy_to,
  * Frees memory previously allocated by e_client_util_strv_to_slist().
  *
  * Since: 3.2
+ *
+ * Deprecated: 3.8: Use g_slist_free_full() instead.
  **/
 void
 e_client_util_free_string_slist (GSList *strings)
@@ -1691,6 +1701,8 @@ e_client_util_free_string_slist (GSList *strings)
  * itself.
  *
  * Since: 3.2
+ *
+ * Deprecated: 3.8: Use g_slist_free_full() instead.
  **/
 void
 e_client_util_free_object_slist (GSList *objects)
