@@ -1,5 +1,5 @@
 /*
- * e-source-registry.c
+ * evolution-source-registry.c
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -31,6 +31,11 @@
 void evolution_source_registry_migrate_basedir (void);
 void evolution_source_registry_migrate_sources (void);
 
+gboolean	evolution_source_registry_migrate_imap_to_imapx
+						(ESourceRegistryServer *server,
+						 GKeyFile *key_file,
+						 const gchar *uid);
+
 gint
 main (gint argc,
       gchar **argv)
@@ -64,6 +69,12 @@ reload:
 
 	server = e_source_registry_server_new ();
 
+	/* Convert "imap" mail accounts to "imapx". */
+	g_signal_connect (
+		server, "tweak-key-file", G_CALLBACK (
+		evolution_source_registry_migrate_imap_to_imapx),
+		NULL);
+
 	/* Failure here is fatal.  Don't even try to keep going. */
 	e_source_registry_server_load_all (
 		E_SOURCE_REGISTRY_SERVER (server), &error);
@@ -93,9 +104,9 @@ reload:
 		g_print ("Reloading...\n");
 
 		/* It's possible the Reload is called after restore, where
-		   the ~/.config/evolution/sources directory can be missing,
-		   thus create it, because e_server_side_source_get_user_dir()
-		   may have its static variable already set to non-NULL value.
+		 * the ~/.config/evolution/sources directory can be missing,
+		 * thus create it, because e_server_side_source_get_user_dir()
+		 * may have its static variable already set to non-NULL value.
 		*/
 		config_dir = e_get_user_config_dir ();
 		dirname = g_build_filename (config_dir, "sources", NULL);
