@@ -22,7 +22,7 @@ free_busy_data_cb (ECalClient *client,
 
 static void
 test_get_free_busy_sync (ETestServerFixture *fixture,
-			 gconstpointer       user_data)
+                         gconstpointer user_data)
 {
 	ECalClient *cal_client;
 	GError *error = NULL;
@@ -55,7 +55,7 @@ async_get_free_busy_result_ready (GObject *source_object,
 {
 	ECalClient *cal_client;
 	GError *error = NULL;
-	GMainLoop *loop = (GMainLoop *)user_data;
+	GMainLoop *loop = (GMainLoop *) user_data;
 
 	cal_client = E_CAL_CLIENT (source_object);
 
@@ -70,7 +70,7 @@ async_get_free_busy_result_ready (GObject *source_object,
 
 static void
 test_get_free_busy_async (ETestServerFixture *fixture,
-			  gconstpointer       user_data)
+                          gconstpointer user_data)
 {
 	ECalClient *cal_client;
 	icaltimezone *utc;
@@ -102,10 +102,12 @@ main (gint argc,
 #endif
 	g_test_init (&argc, &argv, NULL);
 
-	g_test_add ("/ECalClient/GetFreeBusy/Sync", ETestServerFixture, &cal_closure,
-		    e_test_server_utils_setup, test_get_free_busy_sync, e_test_server_utils_teardown);
-	g_test_add ("/ECalClient/GetFreeBusy/Async", ETestServerFixture, &cal_closure,
-		    e_test_server_utils_setup, test_get_free_busy_async, e_test_server_utils_teardown);
+	g_test_add (
+		"/ECalClient/GetFreeBusy/Sync", ETestServerFixture, &cal_closure,
+		e_test_server_utils_setup, test_get_free_busy_sync, e_test_server_utils_teardown);
+	g_test_add (
+		"/ECalClient/GetFreeBusy/Async", ETestServerFixture, &cal_closure,
+		e_test_server_utils_setup, test_get_free_busy_async, e_test_server_utils_teardown);
 
 	return e_test_server_utils_run ();
 }
